unit SepiaTasks;

interface

{$setc REQUIRE_MP := FALSE}

	uses
		Types, Threads, Files, CodeFragments, {}
{$ifc REQUIRE_MP}
		MP, { pour MP Threads }
{$endc}
		BigStrings, {} 
		MyStrings, SepiaNetwork, SepiaVersion, SepiaScript;
	
	const
		kLibraryName = 'Sepia Shared Library';
		kGetNameFunc = 'GetName';
		kGetNameProcInfo = $000000E0;
		kAnalyzeProc = 'Analyze';
		kAnalyzeProcInfo = $000000E0;
		
		kNameTag = 'NAME';
		kParamTag = 'PARAM';
		kSleepTag = 'SLEEP';
		kShutDownTag = 'SHUTDOWN';
		kOKTag = 'OK';
		
		kMaxDownload = 3; { Don't download more than this number of times before sleeping }
		kSleepTicks = 60 * 60 * 30; { 30 minutes }
		kMaxTicksToSleep = 60 * 60 * 60 * 24; { number of ticks in a day }
		MAX_CHAR_LENGTH = 178956970; { a large enough number }

	type
		TCharVector = packed array[1..MAX_CHAR_LENGTH] of Char;
		TCharVectorPtr = ^TCharVector;
		TCharVectorH = ^TCharVectorPtr;
		TSepiaStatus = (task_none, task_checkVersion, task_compareVersion, task_startNewVersion, task_new, task_inProgress, task_done, task_transferInProgress, task_clearing, task_quitting);
		TTimeMeasure = (ticks, seconds, minutes, hours, days);
		TYieldType = (NoYieldTask, MPTask, ThreadTask);
		TSepiaTask = record
			status: TSepiaStatus;
			name: Str255;
			param: BigStringPtr;
		end; { record }
		TSepiaTaskPtr = ^TSepiaTask;
		
	var
		currentTask: TSepiaTask;
		gThreadResult, gSleepUntil: LongInt;
		gDownloadCount: Integer;
		gTaskYield: TYieldType;

	procedure InitTask(var task: TSepiaTask);
	procedure GetNewTaskFromServer(var task: TSepiaTask);
	function HaveAlgorithm(name: Str255): Boolean;
	procedure LaunchTask(var task: TSepiaTask);
	procedure RetrieveLibraryFromServer(var task: TSepiaTask);
	procedure GetVersionFromServer(var task: TSepiaTask);
	procedure UpdateProgress(var task: TSepiaTask);
	procedure SendTaskToServer(var task: TSepiaTask);
	procedure ClearTask(var task: TSepiaTask);
	
	function DifferentVersion(newVer, oldVer: Str255): Boolean;
	function ChangeMyName: Boolean;
	function StartNewVersion: Boolean;
	procedure DeleteOldSepia;
	procedure DownloadNewVersionFromServer(var task: TSepiaTask);
	
	function RandomDelay(max: LongInt; timeUnit: TTimeMeasure): LongInt;

implementation

uses
	Binhex; { pour ConvertFromBinhex }


	function GetCurrentAppFSSpec (var fss: FSSpec): OSErr;
		var
			info: ProcessInfoRec;
			PSN: ProcessSerialNumber;
			err: OSErr;
	begin
		info.processInfoLength := SizeOf(ProcessInfoRec);
		info.processName := nil;
		info.processAppSpec := FSSpecPtr(NewPtr(SizeOf(FSSpec)));
		err := GetCurrentProcess(PSN);
		err := GetProcessInformation(PSN, info);
		if (err = noErr) then
			BlockMove(Ptr(info.processAppSpec), @fss, SizeOf(FSSpec));

		DisposePtr(Ptr(info.processAppSpec));

		GetCurrentAppFSSpec := err;

	end; { GetCurrentAppFSSpec }

{ This function was extracted from the MoonPhase program by Bill Catambay }
	function Byte2Hex(l: Integer): Str255;
	var
		i: Integer;
		hexString: packed array[0..15] of Char;
	begin
		hexString := '0123456789ABCDEF';
		Byte2Hex[0] := chr(2);
		for i := 1 to 2 do
			Byte2Hex[i] := hexString[BAnd(BRotL(l, 24 + BSL(i, 2)), 15)];
	end; { Byte2Hex }
	
	function URLEncode(s: Str255): Str255;
	var
		i, j: Integer;
		r: Str255;
	begin
		r := '';
		j := 1; { keep a count of length of result }
		for i := 1 to Length(s) do begin
			if s[i] in ['0'..'9', 'a'..'z', 'A'..'Z', '$', '-', '_', '.', '+', '!', '*', '''', '(', ')', ','] then begin { no encoding necessary, see RFC 1738 }
				r := Concat(r, s[i]);
				j := j + 1;
			end else begin { must encode in hex }
				r := Concat(r, '%', Byte2Hex(Ord(s[i])));
				j := j + 3;
			end;
			if (j > 254) then
				;
		end;
		URLEncode := r;
	end; { URLEncode }

	procedure BigURLEncode(var b: BigStringPtr);
	var
		i, j: Integer;
		c: BigStringPtr;
	begin
		c := NewBigStringPtr;
		j := 1; { keep a count of length of result }
		for i := 1 to b^.length do begin
			if b^.chr[i] in ['0'..'9', 'a'..'z', 'A'..'Z', '$', '-', '_', '.', '+', '!', '*', '''', '(', ')', ','] then begin { no encoding necessary, see RFC 1738 }
				c^.chr[j] := b^.chr[i];
				j := j + 1;
				c^.length := c^.length + 1;
			end else begin { must encode in hex }
				BigStrConcat(c, StringOf('%', Byte2Hex(Ord(b^.chr[i]))));
				j := j + 3;
			end;
			if (j > kBigStringMaxLength) then
				;
		end;
		DisposeBigStringPtr(b);
		b := c;
	end; { BigURLEncode }

	function BinHexDecode(var fss: FSSpec; deleteOld: Boolean): OSErr;
	var
		err: OSErr;
	begin
		err := ConvertFromBinhex(fss, true, deleteOld, nil, nil);
		BinHexDecode := err;
	end; { BinHexDecode }

	function FindSharedLibrary(name: Str63; var cID: CFragConnectionID; var errN: Str255): OSErr;
	var
		mainAddr: Ptr;
	begin
		mainAddr := nil;
		cID := nil;
		FindSharedLibrary := GetSharedLibrary(name, kPowerPCCFragArch, kPrivateCFragCopy, cID, mainAddr, errN);
	end; { FindSharedLibrary }

	function FindFunction(name: Str255; cID: CFragConnectionID; var addr: Ptr): OSErr;
	var
		symClass: CFragSymbolClass;
	begin
		addr := nil;
		FindFunction := FindSymbol(cID, name, addr, symClass);
	end; { FindFunction }

	function CallGetNameFunction(addr: Ptr; var arg: Str255): OSErr;
	var
		upp: UniversalProcPtr;
		s: StringPtr;
	begin
		upp := NewRoutineDescriptor(ProcPtr(addr), kGetNameProcInfo, GetCurrentISA);
		if (upp <> nil) then begin
			s := @arg;
			CallGetNameFunction := CallUniversalProc(upp, kGetNameProcInfo, s);
			arg := s^;
			DisposeRoutineDescriptor(upp);
		end	else
			CallGetNameFunction := memFullErr;
	end; { CallGetNameFunction }
	
	procedure SearchForTag(buf: TCharVectorH; bt, et: Str255; var bo, eo: LongInt);
	begin
		bo := 0;
		eo := 0;
		{ begin tag }
		bo := Munger(Handle(buf), 0, @bt[1], Length(bt), nil, 0);
		if (bo >= 0) then
			bo := bo + Length(bt) + 1;
		{ end tag }
		eo := Munger(Handle(buf), bo, @et[1], Length(et), nil, 0);
		if (eo > 0) then
			eo := eo + 1;
	end; { SearchForTag }
	
{ This procedure changes all tags (between '<' and '>') to their upper-case }
{ equivalent }
	procedure CapitalizeTags(var buf: TCharVectorH);
		var 
			i, l: LongInt;
			inTag: Boolean;
	begin
		inTag := false;
		l := GetHandleSize(Handle(buf));
		if (l > 0) then
			for i := 1 to l do begin
				if buf^^[i] = '<' then
					inTag := true 
				else if buf^^[i] = '>' then
					inTag := false;
				if inTag then
					if Ord(buf^^[i]) in [97..122] then { 'a' to 'z' }
						buf^^[i] := Chr(Ord(buf^^[i]) - 32) {'A' to Z' }
			end; { for i }
	end; { CapitalizeTags }

{ This function parses a text buffer for the relevant information }
	function ParseTaskBuffer(var pTask: TSepiaTaskPtr; buffer: TCharVectorH): OSErr;
		var
			err: OSErr;
			beginTag, endTag: Str255;
			beginOffset, endOffset, l: LongInt;
	begin
		err := noErr;
		CapitalizeTags(buffer);
		{ Search for name tag }
		beginTag := Concat('<', kNameTag, '>');
		endTag := Concat('</', kNameTag, '>');
		SearchForTag(buffer, beginTag, endTag, beginOffset, endOffset);
		if (beginOffset < endOffset) then begin
			l := (endOffset - beginOffset);
			if (l > 255) then
				l := 255; { force name to be 255 chars long }
			BlockMoveData(@buffer^^[beginOffset], @pTask^.name[1], l);
			pTask^.name[0] := Chr(l);
		end { if (beginOffset < endOffset) }
		else 
			pTask^.name := ''; { tag not found }

		if (noErr = err) then begin
			{ Search for parameter tag }
			beginTag := Concat('<', kParamTag, '>');
			endTag := Concat('</', kParamTag, '>');
			SearchForTag(buffer, beginTag, endTag, beginOffset, endOffset);
			if (beginOffset < endOffset) then begin
				l := (endOffset - beginOffset);
				BlockMoveData(@buffer^^[beginOffset], @pTask^.param^.chr[1], l);
				pTask^.param^.length := l;
			end { if (beginOffset < endOffset) }
			else
				err := -1; { tag not found }
		end;
		
		ParseTaskBuffer := err;
	end; { ParseTaskBuffer }

{ This function parses the downloaded fsspec and fills the task record }
{ with the relevant information }
	function ParseTask(var pTask: TSepiaTaskPtr; var fss: FSSpec): OSErr;
		var
			err, junk: OSErr;
			refNum: Integer;
			curEOF: LongInt;
			buffer: TCharVectorH;
	begin
		err := FSpOpenDF(fss, fsRdPerm, refNum);
		if (noErr = err) then begin
			err := GetEOF(refNum, curEOF);
			if (noErr = err) and (curEOF > 0) then begin { something to work with }
				buffer := TCharVectorH(NewHandleClear(curEOF));
				if (buffer <> nil) then begin
					err := FSRead(refNum, curEOF, Ptr(buffer^));
					if (noErr = err) then 
						err := ParseTaskBuffer(pTask, buffer);
					DisposeHandle(Handle(buffer));
				end else
					err := memFullErr;
			end else begin { curEOF > 0 }
				if (curEOF = 0) then
					err := eofErr;
			end;
			
			junk := FSClose(refNum);
			junk := FSpDelete(fss); { we don't need this file anymore, clean up }
		end; { noErr FSpOpenDF }
		
		ParseTask := err;
	end; { ParseTask }


	function NewTaskFromServer(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		taskURL: BigStringPtr;
		idStr, macName: Str255;
		err: OSErr;
		taskURLPtr: Ptr;
		l, numTicks: LongInt;
		numTicksStr: Str255;
		
	begin
//		EventLog('Attempting to retrieve task from server', true);
		taskURL := NewBigStringPtr;
		StrToBigString(MyGetStr(kServerStrings, kSepiaCGIServerID, 1), taskURL);
		if (taskURL^.length > 0) then begin
			idStr := 'No ID';
			macName := GetMachineName;
			macName := URLEncode(macName);
			
			BigStrConcat(taskURL, StringOf(MyGetStr(kServerStrings, kGetTaskCommand, 1){, '?id=', idStr}));
//			if (macName <> '') then
//				BigStrConcat(taskURL, StringOf('&macName=', macName));
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := 'gettask';
				l := taskURL^.length;
				taskURLPtr := NewPtrClear(l + 1);
				if (taskURLPtr <> nil) then begin
					BlockMoveData(@taskURL^.chr[1], taskURLPtr,l);
//					taskURLPtr^[l] := 0;
					err := DownloadURL(taskURLPtr, nil, fss);
					if (err <> noErr) then
//						EventLog(StringOf('Error ', err:1, ' while downloading task data from server'), true)
					else begin
						if (noErr = ParseTask(pTask, fss)) then begin { if we can find the parameters }
							if kSleepTag = pTask^.name then begin { we are asked to sleep }
								pTask^.status := task_none;
								{ The number of ticks to sleep is in the parameter tag }
								BigStringToStr(pTask^.param, numTicksStr);
								StringToNum(numTicksStr, numTicks);
								{ Ensure that the parameter makes sense... }
								if numTicks > kMaxTicksToSleep then
									numTicks := kMaxTicksToSleep;
								if numTicks < 1 then
									numTicks := kSleepTicks;
								
								gSleepUntil := TickCount + numTicks;
								
//								EventLog(StringOf('Server asked us to sleep for ', (numTicks div 60):1, ' seconds.'), true);
							end else if kShutdownTag = pTask^.name then begin { we are asked to shut down the machine }
								pTask^.status := task_quitting;
//								EventLog('Server asked us to down the machine', true);
								gSleepUntil := 0; { do it now }
								SepiaShutDown;
							end else begin
								pTask^.status := task_new; { we have a new task! }
								gSleepUntil := 0; { do this now! }
//								EventLog(StringOf('Successfully retrieved task ', pTask^.name, ' from server'), true);
//								UpdateTask(pTask^.name, pTask^.param);
							end;
						end else begin
							pTask^.status := task_none;
//							EventLog(StringOf('Could not get information from task ', fss.name, '. The server may be down.'), true);
							numTicks := RandomDelay(2, seconds);
							gSleepUntil := TickCount + numTicks;
//							EventLog(StringOf('Sleeping for ', (numTicks div 60):1, ' seconds.'), true);
						end;
					end;
						
					DisposePtr(taskURLPtr);
				end { if <> nil }
				else;
//					EventLog('Not enough memory to allocate URL!', false);
			end { if noErr }
			else;
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
		DisposeBigStringPtr(taskURL);
		
		NewTaskFromServer := noErr;
	end; { NewTaskFromServer }

	function SendTask(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		taskURL, idStr, resStr: Str255;
		err: OSErr;
		postArgs, saveResult: BigStringPtr;
		taskURLPtr, postArgsPtr: Ptr;
		lt, lp: LongInt;
		
	begin
//		EventLog('Attempting to send results to server', true);
		postArgs := nil;//NewBigStringPtr;
		saveResult := NewBigStringPtr;
		taskURL := MyGetStr(kServerStrings, kSepiaCGIServerID, 1);
		if (Length(taskURL) > 0) then begin
			idStr := 'No ID';
			
//			UpdateTask('Result', pTask^.param);
			BigConcat(saveResult, pTask^.param); // save a copy of the results, in case 
			BigURLEncode(pTask^.param);
			
			taskURL := Concat(taskURL, MyGetStr(kServerStrings, kSendResultCommand, 1));//StringOf('SendResult.acgi', '?id=', idStr, '&result='));
			
//			BigStrConcat(postArgs, StringOf('id=', idStr, '&result='));
//			BigConcat(postArgs, pTask^.param);
			
			
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := 'sendtask';
				lt := Length(taskURL);
				lp := 0;//postArgs^.length;
				taskURLPtr := NewPtrClear(lt + 1);
				postArgsPtr := nil;//NewPtrClear(lp + 1);
				if (taskURLPtr <> nil) {and (postArgsPtr <> nil)} then begin
					BlockMoveData(@taskURL[1], taskURLPtr, lt);
//					taskURLPtr^[l] := 0;
//					BlockMoveData(@postArgs^.chr[1], postArgsPtr, lp);
//					postArgsPtr^[l] := 0;
					err := DownloadURL(taskURLPtr, postArgsPtr, fss);
					if (err <> noErr) then
//						EventLog(StringOf('Error ', err:1, ' while downloading task data from server'), true)
					else begin
						if (noErr = ParseTask(pTask, fss)) then begin { if we can find the parameters }
							if BigStrEqual(pTask^.param, kOKTag) then begin { everything went well }
								{ Reset everything }
								gSleepUntil := 0;
								pTask^.status := task_none;
//								if (noErr <> GenerateUUID(gUUID)) then
//									UpdateUUID; { generate new id for the next transaction }
								
//								EventLog('Results sent OK', true);
							end else begin { start again }
								BigStringToStr(pTask^.param, resStr);
//								EventLog(StringOf('Error from server: ', resStr), true);
								pTask^.status := task_done; { start again }
								pTask^.param^.length := 0; { clear parameters }
								BigConcat(saveResult, pTask^.param); { Get saved copy of results }
							end;
						end else begin
							pTask^.status := task_none;
//							EventLog(StringOf('No result from task ', fss.name, '. The server may be down.'), true);
						end;
					end;
						
					DisposePtr(taskURLPtr);
//					DisposePtr(postArgsPtr);
				end { if <> nil }
				else;
//					EventLog('Not enough memory to allocate URL!', false);
			end { if noErr }
			else;
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
//		DisposeBigStringPtr(postArgs);
		DisposeBigStringPtr(saveResult);
		
		SendTask := noErr;
	end; { SendTask }

	function DownloadLibrary(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		taskURL: Str255;
		err: OSErr;
		taskURLPtr: Ptr;
		l: LongInt;
	begin
//		EventLog('Attempting to retrieve library from server', true);
		taskURL := MyGetStr(kServerStrings, kSepiaWebServerID, 1);
//		taskURL := 'http://biol10.biol.umontreal.ca/casgrain/';
		if (taskURL <> '') then begin
			taskURL := Concat(taskURL, pTask^.name, MyGetStr(kServerStrings, kSepiaBinhexSuffix, 1));
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := 'Shared Library.hqx';
				l := Length(taskURL);
				taskURLPtr := NewPtrClear(l + 1);
				if (taskURLPtr <> nil) then begin
					BlockMoveData(@taskURL[1], taskURLPtr,l);
//					taskURLPtr^[l] := 0;
					err := DownloadURL(taskURLPtr, nil, fss);
					if (err <> noErr) then
//						EventLog(StringOf('Error ', err:1, ' while downloading library from server'), true)
					else begin
//						EventLog('Successfully retrieved new library from server', true);
						err := BinHexDecode(fss, true);
						if (noErr = err) then begin
							gDownloadCount := gDownloadCount + 1;
							if (gDownloadCount < kMaxDownload) then begin
								pTask^.status := task_new; { back to the new task }
								gSleepUntil := RandomDelay(2, seconds); { do this now! }
//								EventLog(StringOf('Successfully installed new library ', fss.name, ' from server'), true);
							end;
						end else begin
//							EventLog(StringOf('Error ', err:1, ' installing new library'), true);
						end;
					end;
						
					DisposePtr(taskURLPtr);
				end { if <> nil }
				else
//					EventLog('Not enough memory to allocate URL!', false);
			end { if noErr }
			else
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
		DownloadLibrary := err;
	end; { DownloadLibrary }

	function DownloadNewVersion(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		name, taskURL: Str255;
		err: OSErr;
		taskURLPtr: Ptr;
		l: LongInt;
	begin
		taskURL := MyGetStr(kServerStrings, kSepiaWebServerID, 1);
		if (taskURL <> '') then begin
			name := Concat(MyGetStr(kServerStrings, kSepiaName, 1), MyGetStr(kServerStrings, kSepiaBinhexSuffix, 1));
			taskURL := Concat(taskURL, name);
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := name;
				l := Length(taskURL);
				taskURLPtr := NewPtrClear(l + 1);
				if (taskURLPtr <> nil) then begin
					BlockMoveData(@taskURL[1], taskURLPtr,l);
					err := DownloadURL(taskURLPtr, nil, fss);
					if (err <> noErr) then
//						EventLog(StringOf('Error ', err:1, ' while downloading application from server'), true)
					else begin
//						EventLog('Successfully retrieved new version from server', true);
						err := BinHexDecode(fss, true);
						if (noErr = err) then begin
//							EventLog(StringOf('Successfully installed new version from server'), true);
							pTask^.status := task_startNewVersion;
							gSleepUntil := 0; { do this now! }
						end else begin
//							EventLog(StringOf('Error ', err:1, ' installing new version'), true);
							pTask^.status := task_none;
						end;
					end;
					DisposePtr(taskURLPtr);
				end { if <> nil }
				else
//					EventLog('Not enough memory to allocate URL!', false);
			end { if noErr }
			else
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
		DownloadNewVersion := err;
	end; { DownloadNewVersion }


	function GetVersion(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		taskURL: BigStringPtr;
		err: OSErr;
		taskURLPtr: Ptr;
		l: LongInt;
		
	begin
		taskURL := NewBigStringPtr;
		StrToBigString(MyGetStr(kServerStrings, kSepiaWebServerID, 1), taskURL);
		if (taskURL^.length > 0) then begin			
			BigStrConcat(taskURL, MyGetStr(kServerStrings, kVersionText, 1));
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := MyGetStr(kServerStrings, kVersionText, 1);
				l := taskURL^.length;
				taskURLPtr := NewPtrClear(l + 1);
				if (taskURLPtr <> nil) then begin
					BlockMoveData(@taskURL^.chr[1], taskURLPtr,l);
					err := DownloadURL(taskURLPtr, nil, fss);
					if (err <> noErr) then
//						EventLog(StringOf('Error ', err:1, ' while downloading version info from server'), true)
					else begin
						err := ParseTask(pTask, fss);
						if (Length(pTask^.name) > 0) then
							err := noErr;
						if (noErr = err) then begin { if we can find a version }
							pTask^.status := task_compareVersion; { change thing to do }
							gSleepUntil := 0; { do this now! }
						end else begin
							pTask^.status := task_none;
//							EventLog(StringOf('Could not get version info from server'), true);
						end;
					end;
					DisposePtr(taskURLPtr);
				end { if <> nil }
				else
//					EventLog('Not enough memory to allocate URL!', false);
			end { if noErr }
			else
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
		DisposeBigStringPtr(taskURL);
		
		GetVersion := noErr;
	end; { GetVersion }

	function StartTask(pTask: TSepiaTaskPtr): LongInt;
	var
		err, status: OSErr;
		connID: CFragConnectionID;
		symAddr: Ptr;
		errName: Str255;
		upp: UniversalProcPtr;
{$ifc REQUIRE_MP}
		mouseRgn: RgnHandle;
		communicationQueue: MPQueueID;(* This queue will be used to communicate between the app and the task. *)
		terminationQueue: MPQueueID;		(* This queue will report the completion of the task. *)
		taskID: MPTaskID;						(* This will be the ID of the task we create. *)
{$endc}
		taskDone: Boolean;
		result: LongInt;
		threadMade: ThreadID;
	begin
		case gTaskYield of
			NoYieldTask:;
//				EventLog('Launching task', true);
			MPTask:;
//				EventLog('Launching MP task', true)
			ThreadTask:;
//				EventLog('Launching Thread task', true);
			otherwise;
//				EventLog('Unable to launch task. Yield type unknown.', true);
		end; { case }
		
		
		{ Load the shared library }
		err := FindSharedLibrary(kLibraryName, connID, errName);
		if (err = noErr) then begin
			err := FindFunction(kAnalyzeProc, connID, symAddr);
			if (err = noErr) then begin

				case gTaskYield of
					NoYieldTask: begin { old way }
						upp := NewRoutineDescriptor(ProcPtr(symAddr), kAnalyzeProcInfo, GetCurrentISA);
						if (upp <> nil) then begin
							err := CallUniversalProc(upp, kAnalyzeProcInfo, pTask^.param);
							DisposeRoutineDescriptor(upp);
						end	else
							err := memFullErr;
					end; { NoYieldTask }
					
					MPTask: begin { New way with MPTasks }
{$ifc REQUIRE_MP}
						mouseRgn := NewRgn;

						err := MPCreateQueue(terminationQueue);	(* Create the queue which will report the completion of the task. *)
						if (err <> noErr) then
							DebugStr('Cannot create the termination queue');
						if err = noErr then
							err := MPCreateQueue(communicationQueue);	(* Create the queue we'll use to communicate with. *)
						if (err <> noErr) then begin
							err := MPDeleteQueue(terminationQueue);
							DebugStr('Cannot create the communication queue');
						end;
						
						status := MPCreateTask(symAddr, pTask^.param, kMPUseDefaultStackSize, terminationQueue, nil, nil, kMPNormalTaskOptions, taskID);
						if (status <> noErr) then begin
							DebugStr('Error creating task');
						end;

						{ Wait for the task to complete: }
						taskDone := false;
						while not taskDone do begin
							taskDone := true;

							status := MPWaitOnQueue(terminationQueue, nil, nil, nil, kDurationImmediate);
							if status = kMPTimeOutErr then begin
								status := noErr; { it's not really an error... }
								taskDone := false;
							end; { kMPTimeOutErr }
							if (status <> noErr) then
								DebugStr('While waiting for task completion');
							
							status := YieldToAnyThread;

						end; { while not taskDone }

						DisposeRgn(mouseRgn);
						
						err := status;
{$endc}
					end; { MPTask }
					
					ThreadTask: begin { Use the Thread Manager; the task must yield explicitely }
						result := 1;
						err := NewThread(kCooperativeThread, ThreadEntryTPP(symAddr), pTask^.param, 0, kCreateIfNeeded, result, threadMade);
						{ Wait for the task to complete: }
						taskDone := false;
						while (result > 0) do begin
							status := YieldToAnyThread;
						end; { while result }
					end; { ThreadTask }
					
					otherwise
						;
				end; { case }
		
			end;
			err := CloseConnection(connID);
		end;
		
		pTask^.status := task_done;
		gSleepUntil := 0;
		if (noErr = err) then begin
//			EventLog('Task completed', true);
		end else begin
//			EventLog(StringOf('Task error ', err:1), true);
		end;
		
		StartTask := err;
	end; { StartTask }

	function LaunchThread(proc: ProcPtr; pTask: TSepiaTaskPtr): OSErr;
	var
		err: OSErr;
		threadMade: ThreadID;
	begin
		err := NewThread(kCooperativeThread, ThreadEntryTPP(proc), Ptr(pTask), 0, kCreateIfNeeded, gThreadResult, threadMade);
		LaunchThread := err;
	end; { LauchThread }

	procedure InitTask(var task: TSepiaTask);
	begin
		task.status := task_checkVersion;
		task.name := '';
		task.param := NewBigStringPtr;
	end; { InitTask }
	
	procedure GetNewTaskFromServer(var task: TSepiaTask);
	begin
		if (LaunchThread(@NewTaskFromServer, @task) = noErr) then
			task.status := task_transferInProgress;
	end; { GetNewTaskFromServer }

	function HaveAlgorithm(name: Str255): Boolean;
	var
		errName: Str255;
		connID: CFragConnectionID;
		err: OSErr;
		fName: Str255;
		symAddr: Ptr;
		result: Boolean;
	begin
		result := false;
		err := FindSharedLibrary(kLibraryName, connID, errName);
		if (err = noErr) then begin
			err := FindFunction(kGetNameFunc, connID, symAddr);
			if (err = noErr) then begin
				err := CallGetNameFunction(symAddr, fName);
				if (err = noErr) then
					result := (name = fName);
			end;
			err := CloseConnection(connID);
		end;
		if not result then;
//			EventLog(Concat('Function name not found (', fName,').'), true);
		HaveAlgorithm := result;
	end; { HaveAlgorithm }

	procedure LaunchTask(var task: TSepiaTask);
	begin
		if (LaunchThread(@StartTask, @task) = noErr) then
			task.status := task_inProgress;
	end; { LaunchTask }

	procedure RetrieveLibraryFromServer(var task: TSepiaTask);
	begin
		if gDownloadCount < kMaxDownload then begin
			if (LaunchThread(@DownloadLibrary, @task) = noErr) then
				task.status := task_transferInProgress;
		end else begin
//			EventLog('Download count exceeded. Sleeping.', true);
			gSleepUntil := gSleepUntil + kSleepTicks + RandomDelay(2, seconds);
		end;
	end; { RetrieveLibraryFromServer }
	
	procedure GetVersionFromServer(var task: TSepiaTask);
	begin
		if (LaunchThread(@GetVersion, @task) = noErr) then
			task.status := task_transferInProgress;
	end; { GetVersionFromServer }
	

	procedure UpdateProgress(var task: TSepiaTask);
	begin
		task.status := task_inProgress;
	end; { UpdateProgress }

	procedure SendTaskToServer(var task: TSepiaTask);
	begin
		if (LaunchThread(@SendTask, @task) = noErr) then
			task.status := task_transferInProgress;
	end; { SendTaskToServer }

	function ClearTaskFromServer(pTask: TSepiaTaskPtr): LongInt;
	var
		fss: FSSpec;
		taskURL, idStr: Str255;
		err, junk: OSErr;
		taskURLPtr: Ptr;
		l: LongInt;
		
	begin
//		EventLog('Attempting to delete task from server', true);
		taskURL := MyGetStr(kServerStrings, kSepiaCGIServerID, 1);
		if (taskURL <> '') then begin
//			err := ConvertUUIDToString(gUUID, idStr);
			
			taskURL := Concat(taskURL, MyGetStr(kServerStrings, kClearDataCommand, 1){, '?id=', idStr, '&resultsOnly=false'});
			err := GetCurrentAppFSSpec(fss);
			if (err = noErr) then begin
				fss.name := 'cleardata';
				l := Length(taskURL);
				taskURLPtr := NewPtrClear(l + 1);
				if (taskURLPtr <> nil) then begin
					BlockMoveData(@taskURL[1], taskURLPtr,l);
//					taskURLPtr^[l] := 0;
					err := DownloadURL(taskURLPtr, nil, fss);
					if (err <> noErr) then;
//						EventLog(StringOf('Error ', err:1, ' while clearing task data from server'), true);						
					DisposePtr(taskURLPtr);
				end { if <> nil }
				else;
//					EventLog('Not enough memory to allocate URL!', false);
				junk := FSpDelete(fss);
			end { if noErr }
			else
//				EventLog('Could not find this app''s FSSpec!', false);
		end { if <> '' }
		else;
//			EventLog('Server string not found!', false);
		
		pTask^.status := task_quitting;
		
		if (noErr = err) then;
//			EventLog('Successfully deleted task from server', true);
		
		ClearTaskFromServer := err;
	end; { ClearTaskFromServer }
	
	procedure ClearTask(var task: TSepiaTask);
	begin
		if (LaunchThread(@ClearTaskFromServer, @task) = noErr) then
			task.status := task_clearing;
	end; { ClearTask }
	
	function DifferentVersion(newVer, oldVer: Str255): Boolean;
	begin
		{ Bypass check if it's a development version }
		if (oldVer = kSepiaDontCheckVersion) or (newVer = kSepiaDontCheckVersion) then
			DifferentVersion := false 
		else
			DifferentVersion := (newVer <> oldVer);
	end; { DifferentVersion }
	
	function ChangeMyName: Boolean;
	var
		err: OSErr;
		fss: FSSpec;
		name: Str255;
	begin
		err := GetCurrentAppFSSpec(fss);
		if (noErr = err) then begin
			name := Concat(MyGetStr(kServerStrings, kSepiaName, 1), MyGetStr(kServerStrings, kSepiaDeleteSuffix, 1));
			err := FSpRename(fss, name);
		end; { noErr }
		ChangeMyName := (noErr = err);
	end; { ChangeMyName }

	function StartNewVersion: Boolean;
	var
		myPSN: ProcessSerialNumber;
		myErr: OSErr;
		myAddress: AEDesc;
		launchApp: FSSpec;
		launchThis: LaunchParamBlockRec;
	begin
	    { get the app to launch, using the Sys7 versions of Standard File }
		{ which return FSSpecs }
		{ Preset our address descriptor to our PSN }
		{ The Finder is going to change this to whatever is appropriate  }
		{ for the application that eventually gets launched, so I'll }
		{ just fill it with my own address. }
		{ I'm doing this because I have heard sporadic reports that  }
		{ _not_ doing this causes address errors sometimes, I've never had that happen. }
		{ But since it has, you should prefill. }
		myErr := GetCurrentProcess(myPSN);
		{ create the address desc for the event }
	    myErr := AECreateDesc(typeProcessSerialNumber, @myPSN, SizeOf(ProcessSerialNumber), myAddress);
		{ get the application to launch }
		myErr := GetCurrentAppFSSpec(launchApp);
	    launchApp.name := MyGetStr(kServerStrings, kSepiaName, 1);
		{ stuff it in my launch parameter block }
	    launchThis.launchAppSpec := @launchApp;
	    
	    launchThis.launchBlockID := extendedBlock;
	    launchThis.launchEPBLength := extendedBlockLen;
	    launchThis.launchFileFlags := 0;
	    launchThis.launchControlFlags := launchContinue + launchNoFileFlags;
	    myErr := LaunchApplication(@launchThis);
	    { and launch it }
	    
		StartNewVersion := (noErr = myErr);
	end; { StartNewVersion }

	procedure DeleteOldSepia;
	var
		err: OSErr;
		fss: FSSpec;
		name: Str255;
	begin
		err := GetCurrentAppFSSpec(fss);
		if (noErr = err) then begin
			name := Concat(MyGetStr(kServerStrings, kSepiaName, 1), MyGetStr(kServerStrings, kSepiaDeleteSuffix, 1));
			fss.name := name;
			err := FSpDelete(fss);
		end; { if noErr }
	end; { DeleteOldSepia }

	procedure DownloadNewVersionFromServer(var task: TSepiaTask);
	begin
		if (LaunchThread(@DownloadNewVersion, @task) = noErr) then
			task.status := task_transferInProgress;
	end; { DownloadNewVersionFromServer }
	
	function RandomDelay(max: LongInt; timeUnit: TTimeMeasure): LongInt;
	var
		r: Integer;
		d: double;
	begin
		if timeUnit > ticks then
			max := max * 60;
		if timeUnit > seconds then
			max := max * 60;
		if timeUnit > minutes then
			max := max * 60;
		if timeUnit > hours then
			max := max * 24;
			
		r := Abs(Random);
		d := r;
		d := d * max;
		d := d / MaxInt;

		RandomDelay := System.Trunc(d);
	end; { RandomDelay }

end.