//
// Confirmania: Because you can never be to safe
//
//  2001 Chris Page <page@best.com>
//

#include <Dialogs.h>
#include <Menus.h>
#include <MixedMode.h>
#include <Patches.h>
#include <QuickDraw.h>
#include <Traps.h>

enum
{
	kStrID	= 128,
	kStrCount = 3
};

static Str255			alertMessage[kStrCount];
static Str255			alertDetails[kStrCount];

static UniversalProcPtr	macosModalDialogUPP;

#define uppModalDialogInfo	kPascalStackBased													\
							| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(ModalFilterUPP)))		\
							| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(DialogItemIndex*)))

static pascal void ConfirmaniaModalDialogPatch( ModalFilterUPP modalFilter, DialogItemIndex *itemHit )
{
	static Boolean	fAlreadyEnteredPatch = false;
	GrafPtr			thePort;
	
	CALL_TWO_PARAMETER_UPP( macosModalDialogUPP, uppModalDialogInfo, modalFilter, itemHit );
	
	// Prevent re-entrancy. This will get called while inside of StandardAlert(), below.
	
	if (fAlreadyEnteredPatch)
	{
		return;
	}
	
	fAlreadyEnteredPatch = true;
	
	GetPort( &thePort );
	
	if (*itemHit == GetDialogDefaultItem( thePort ))
	{
		// Display a useless and annoying confirmation alert.
		
		OSErr					err;
		AlertStdAlertParamRec	alertParams;
		int						messageIndex = 0;
		SInt16					alertItemHit;
		
		alertParams.movable = false;
		alertParams.helpButton = false;
		alertParams.filterProc = NULL;
		alertParams.defaultText = (ConstStringPtr) kAlertDefaultOKText;
		alertParams.cancelText = (ConstStringPtr) kAlertDefaultCancelText;
		alertParams.otherText = (ConstStringPtr) NULL;
		alertParams.defaultButton = kAlertStdAlertOKButton;
		alertParams.cancelButton = kAlertStdAlertCancelButton;
		alertParams.position = kWindowDefaultPosition;
		
		do
		{
			SysBeep( 20 );
			
			err = StandardAlert(	kAlertCautionAlert,
									alertMessage[messageIndex],
									alertDetails[messageIndex],
									&alertParams,
									&alertItemHit );
			
			// If the user canceled, return a null event, which will leave the original dialog open.
			
			if (err == noErr && alertItemHit == kAlertStdAlertCancelButton)
			{
				*itemHit = -1;
				break;
			}
			
			// Randomly, keep asking until all messages are displayed.
			
			messageIndex++;
			if (messageIndex >= kStrCount)
			{
				break;
			}
			
		} while ((TickCount() & 1) == 0);
	}
	
	fAlreadyEnteredPatch = false;
}

void main(void)
{
	Handle				myINITHandle;
	THz					theZone;
	UniversalProcPtr	modalDialogPatchUPP;
	
	myINITHandle = Get1Resource( 'INIT', 0 );
	
	if (myINITHandle == NULL)
	{
		return;
	}
	
	DetachResource( myINITHandle );
	if (ResError() != noErr)
	{
		return;
	}
	
	theZone = GetZone();
	SetZone( SystemZone() );
	
	HLockHi( myINITHandle );
	
	modalDialogPatchUPP = NewRoutineDescriptor(	(ProcPtr) ConfirmaniaModalDialogPatch,
												uppModalDialogInfo,
												GetCurrentISA() );
	if (modalDialogPatchUPP == NULL)
	{
		return;
	}
	
	macosModalDialogUPP = GetToolboxTrapAddress( _ModalDialog );
	SetToolboxTrapAddress( modalDialogPatchUPP, _ModalDialog );
	
	{
		int i;
		
		for (i = 0; i < kStrCount; i++)
		{
			GetIndString( alertMessage[i], kStrID, i * 2 + 1 );
			GetIndString( alertDetails[i], kStrID, i * 2 + 2 );
		}
	}
}
