/*
	File:		MoreAppleEventsPlus.h

	Contains:	Functions to help you when you are building and sending Apple events.

	Written by:	George Warner

	Copyright:	Copyright (c) 2000 Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <1>      3/9/00    GW      Intergrating AppleEvent Helper code. First Check In
*/

#pragma once

///********************************************************************************
//	A private conditionals file to setup the build environment for this project.
#include "MoreSetup.h"
//********************************************************************************
//	System includes
#include <AERegistry.h>
#include <AEObjects.h>
#include <AEPackObject.h>
#include <EPPC.h>
#include <Gestalt.h>
#include <Processes.h>
//********************************************************************************
#ifdef __cplusplus
	extern "C" {
#endif __cplusplus
//**********	Private Definitions		****************************************
//*******************************************************************************
#pragma mark Typedefs, structs, enums, etc. 
//*******************************************************************************

typedef struct MoreAEHandler_Struct {
	const AEEventClass			fAEEventClass;
	const AEEventID				fAEEventID;
	const AEEventHandlerProcPtr	fAEEventHandlerProcPtr;
	AEEventHandlerUPP 			fOldAEEventHandlerUPP;
	SInt32						fOldRefCon;
} MoreAEHandlerRec;

typedef struct MoreAEAccessor_Struct {
	const DescType				fDesiredClass;
	const DescType				fContainerType;
	const OSLAccessorProcPtr	fOSLAccessorProcPtr;
	OSLAccessorUPP				fOldOSLAccessorUPP;
	SInt32						fOldRefCon;
} MoreAEAccessorRec;

//*******************************************************************************
// ProcPtr's for our Get/Set Data routines
//*******************************************************************************
typedef pascal OSErr (*MoreAEHandle_GetDataProcPtr)(const AEDesc* pObjAEDesc, AEDesc* pDataAEDesc);
typedef pascal OSErr (*MoreAEHandle_SetDataProcPtr)(const AEDesc* pObjAEDesc, AEDesc* pDataAEDesc);
//*******************************************************************************
extern pascal OSErr MoreAEInitialize(
	const MoreAEHandle_GetDataProcPtr pMoreAEHandle_GetDataProcPtr,
	const MoreAEHandle_SetDataProcPtr pMoreAEHandle_SetDataProcPtr
);

//*******************************************************************************
#pragma mark ==> AEhandlers 
//*******************************************************************************
// This routine installs all the AppleEvent handlers used by this application.
extern pascal void MoreAEInstallAEHandlers(MoreAEHandlerRec* pMoreAEHandlerRecs,const SInt32 pCount);

//*******************************************************************************
// this is how we call thru to any previously installed AEHandler.
extern pascal OSErr MoreAECallPreviousAEHandler(
						const MoreAEHandlerRec* pMoreAEHandlerRecs,
						const AEEventClass	pAEEventClass,
						const AEEventID		pAEEventID,
						const AppleEvent	*pAppleEvent,
						AppleEvent 			*pAEReply,
						UInt32 				pRefcon);
//*******************************************************************************
// call this to install your AEAccessors
extern pascal void MoreAEInstallAEAccessors(MoreAEAccessorRec* pMoreAEAccessorRecs,const SInt32 pCount);
//*******************************************************************************
// this routine calls thru to a previously installed OSLAccessor
extern pascal OSErr MoreAECallPreviousOSLAccessor(
									const MoreAEAccessorRec *pMoreAEAccessorRecs,
									DescType pDesiredClass, 
									const AEDesc* pContainerToken, 
									DescType pContainerClass, 
									DescType pKeyForm, 
									const AEDesc* pKeyData, 
									AEDesc* pToken, 
									long pRefcon);
//*******************************************************************************
// This is an accessor that gets called if a more specific accessor
// isn't found.  (Great for debugging!)
extern pascal OSErr MoreAEAccessWIldFromWild(
									DescType pDesiredClass, 
									const AEDesc* pContainerToken, 
									DescType pContainerClass, 
									DescType pKeyForm, 
									const AEDesc* pKeyData, 
									AEDesc* pToken, 
									long pRefcon);
//*******************************************************************************
// this is our get data AppleEvent handler.
extern pascal OSErr MoreAEHandleGetDataAEvent(
						const AppleEvent *pAppleEvent,
						AppleEvent *pAEReply,
						UInt32 pRefcon);

//*******************************************************************************
// this is our set data AppleEvent handler.
extern pascal OSErr MoreAEHandleSetDataAEvent(
						const AppleEvent *pAppleEvent,
						AppleEvent *pAEReply,
						UInt32 pRefcon);

//*******************************************************************************
// This is a typeWild event hander that will catch any AppleEvents
// that we haven't installed handers for. (Great for debugging!)
extern pascal OSErr MoreAEHandleWildAEvent(
						const AppleEvent *pAppleEvent,
						AppleEvent *pAEReply,
						UInt32 pRefcon);
//********************************************************************************
// Append this property (& it's data) to a list
extern pascal OSErr MoreAEAddPropertyToList(const DescType pPropDescType, AEDesc* pListAEDesc);

#ifdef __cplusplus
	}
#endif __cplusplus
