//
//  Mentat.java
//

/*
	This file and its intellectual contents are considered property of the creator and are to be 	treated as such with respect to use in other applications.  Any use of this software for any 	purpose whatsoever must have explicit permission from the author of the file.  This code is 	part of an ongoing development process for future possible products, and so is considered 	private property.
	Do not use without permission.  Author: Scott C. Ziegler <Aslan@Narnia.net>
*/


import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.util.*;


public class Mentat {

	public static final String LIST_COMMANDS = "?, about, addEntry, addEntryFromFile, removeEntry, lookupEntry, listEntries, changeName, saveArchive, loadArchive, Quit";
    
    // Members
    
    boolean 			interactionMode;
    boolean				touched;
    
    InteractionFrame 	mentatFrame;
    Archive				archie;
    
    FileInputStream		fis;
    FileOutputStream	fos;

    // Constructors
    
    public Mentat() {
		mentatFrame = new InteractionFrame();
        archie = new Archive();
		interactionMode = true;
        }
        
    // Selectors
    
    public Archive getArchie() { return archie; }
    public boolean getInteractionMode() { return interactionMode; }
    
    // Mutators
    
    public void setArchie(Archive newArchie) { newArchie = new Archive(); }
    public void setInteractionMode(boolean mode) { interactionMode = mode; }
    
    // Methods
    
    public void mainline() {
        String selector;
        
        while(true) {
            selector = mentatFrame.getLine();
            
                 if (selector.compareTo("") == 0) {
                    mentatFrame.printEndLine("Enter a command, Moron!");
                    }
            else if (selector.compareTo("help") == 0) {
                    mentatFrame.printLine("Mentat Thufir Howat, at your service.");
                    mentatFrame.print("Available commands: ");
                    mentatFrame.printEndLine(LIST_COMMANDS);
                    }
            else if (selector.compareTo("Commands?") == 0 || selector.compareTo("?") == 0) {
                    mentatFrame.printEndLine(LIST_COMMANDS);
                    }
            else if (selector.compareTo("about") == 0) {
                    about();
                    }
            else if (selector.compareTo("addEntry") == 0) {
                    addEntry();
                    }
	    	else if (selector.compareTo("addEntryFromFile") == 0) {
                    addEntryFromFile();
                    }
            else if (selector.compareTo("removeEntry") == 0) {
                    removeEntry();
                    }
            else if (selector.compareTo("lookupEntry") == 0) {	
                    lookupEntry();
                    }
            else if (selector.compareTo("listEntries") == 0) {
                    listEntries();
                    }
            else if (selector.compareTo("changeName") == 0) {
                    changeName();
                    }
            else if (selector.compareTo("saveArchive") == 0) {
                    saveArchive();
                    }
            else if (selector.compareTo("loadArchive") == 0) {
                    loadArchive();
                    }
            else if (selector.compareTo("Quit") == 0) {	
                    mentatFrame.printLine("Exiting Shell... Quiessence.");
                    System.exit(0);
                    break;
                    }
            else { // default case
                    mentatFrame.printEndLine("The command \""+ selector +"\" was not recognized!");
                    }
           		}
            }
            
    public void about() {
		mentatFrame.printLine("Mentat for MacHack 2001");
		mentatFrame.printLine("written by Scott C. Ziegler, 2000-2001");
		mentatFrame.printLine("presented at MacHack 2001 on MacOS X");
		mentatFrame.printLine("     This small application was developed during experimentations with the Java programming language and its Java Foundation Classes, including Swing.  This is for all those mental notes that always seem to get lost in the cobwebs behind all the deprecated APIs.  This app is dedicated to my friends from days of old when magick filled the aire, and to those who have ever felt the age-smoothed edges of a polyhedron as it decided the fate of some hapless character.");
		mentatFrame.printEndLine("You never saw this fnord.");
		}
            
    public void addEntry() {
        String keyString = new String("");
        String valueString = new String("");
        
        mentatFrame.printLine("Adding an Entry:");
        
        // key variable acquisition
        keyString = getLineLoop("Enter the desired key (i.e. keyword)...", 
        						"That is an invalid key!");
        // value variable acquisition
        valueString = getLineLoop("Enter the desired value associated with this key...",
        							"That value is rather silly, don't you think?");
        
        archie.addEntry(keyString, valueString);
        mentatFrame.printEndLine("Entry Added.");
        touched = true;
        
        // if () { }  // check for entry's existence? then report?
        
        }
        
    public void addEntryFromFile() {
        String filename = new String("");
        String key = new String("");
        mentatFrame.printLine("Adding and Entry from file:");
        key = getLineLoop("Enter the desired key (i.e. keyword)...", 
        						"That is an invalid key!");
        filename = getLineLoop("Enter the filename of the value for the entry.", 
        						"That is an invalid filename!");
        try {
        	FileReader in = new FileReader(filename);
        	BufferedReader bf = new BufferedReader(in);
        	
        	archie.addEntry(key, bf.readLine());
        	mentatFrame.printEndLine("Entry Added.");
        	touched = true;
        	}
        catch (IOException ioe) {
        	System.out.println("Error: " + ioe.toString());
        	}
        }
        
    public void removeEntry() {
        String key = new String("");
        mentatFrame.printLine("Removing an Entry:");
        key = getLineLoop("Enter the key of the entry to be deleted:", "Can't destroy nothing, silly!");
        // Add are you sure dialog text.
       	archie.removeEntry(key);
	mentatFrame.printEndLine("Entry removed.");
       	touched = true;
        }
    
    public void lookupEntry() {
        String key = new String("");
        mentatFrame.printLine("Lookup an Entry:");
        key = getLineLoop("Enter the key of the entry to be looked up:", 
        					"That is not a valid entry fnord.");
        mentatFrame.printEndLine(archie.lookupValue(key));
        }
        
    public void listEntries() {
    	Enumeration e = archie.getArch().keys();
    	while(e.hasMoreElements()) {
    		mentatFrame.printLine((String)e.nextElement());
    		}
    	mentatFrame.printEndLine("There are " + archie.getArch().size() + " entries in the Archive.");
    	}
    
    public void changeName() {
    	String newName = new String("");
    	newName = getLineLoop("Enter the new name of the Archive.", "That's not very imaginative!");
    	archie.setName(newName);
    	mentatFrame.printEndLine("The new name is " + newName + " .");
    	touched = true;
    	}
    
    public void saveArchive() {
        try {
            String filename = new String("untitled.arc");
            filename = getLineLoop("Enter the filename of the Archive file.",
                                                            "That is not a valid filename.");
            fos = new FileOutputStream(filename);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            archie.saveArch(oos);
            mentatFrame.printEndLine("Archive Saved.fnord");
            touched = false;
            }
        catch (FileNotFoundException fnfe) {
            System.out.println("Error: " + fnfe.toString());
            }
        catch (IOException ioe) {
            System.out.println("Error: " + ioe.toString());
            }
    	}
    	
    private void loadBody() {
    	try {
			String filename = new String("untitled.arc");
			filename = getLineLoop("Enter the filename of the Archive file.", "That is not a valid filename.");
			fis = new FileInputStream(filename);
			ObjectInputStream ois = new ObjectInputStream(fis);
			archie.loadArch(ois);
			touched = false;
			mentatFrame.printEndLine("Archive Loaded.");
			}
		catch (FileNotFoundException fnfe) { System.out.println("Error: " + fnfe.toString()); }
		catch (IOException ioe) { System.out.println("Error: " + ioe.toString()); }
    	}
    	
    public void loadArchive() {
		String answer = new String("");
		if (touched) {
			answer = getLineLoop("File not saved! Load anyways?", "Come Again?");
			if (answer.compareTo("Y") == 0 || answer.compareTo("y") == 0
					|| answer.compareTo("Yes") == 0 || answer.compareTo("yes") == 0) loadBody();
			}
		else loadBody();
    	}
    
   	private String getLineLoop(String request, String error) {
   		String temp = new String();
   		while(true) {
        	mentatFrame.printEndLine(request);
        	temp = mentatFrame.getLine();
        	if (temp.compareTo("") == 0) {
        		mentatFrame.printEndLine(error);
        		continue;
        		}
                break;
        	}
        return temp;
   		}
    
    public static void main(String args[]) {
        Mentat thufir = new Mentat();
        thufir.mainline();
        }
    }
