/*
	File:		iTVPlugIn.c

	Contains:	visual plug-in for iTunes

	Written by:	Developer Technical Support

	Copyright:	Copyright 2001 Apple Computer,Inc.

	Change History (most recent first):

         <1>     4/17/01    DTS     first checked in.
*/

#ifdef ACCESSOR_CALLS_ARE_FUNCTIONS
#undef ACCESSOR_CALLS_ARE_FUNCTIONS
#endif
#define ACCESSOR_CALLS_ARE_FUNCTIONS 1	// we use CarbonAccessors.o

/**\
|**|	conditional compilation directives
\**/

/**\
|**|	includes
\**/

#include <ASRegistry.h>
#include <Controls.h>
#include <ControlDefinitions.h>
#include <Lists.h>
#include <Resources.h>
#include <PLStringFuncs.h>

#include <cstring>

#include "MoreAppleEvents.h"
//#include "QDMP.h"
#include "iTunesVisualAPI.h"

/**\
|**|	typedef's, struct's, enum's, etc.
\**/

#define kTVisualPluginName				"\piTVPlugIn"
#define	kTVisualPluginCreator			'hook'

#define	kTVisualPluginMajorVersion		1
#define	kTVisualPluginMinorVersion		0
#define	kTVisualPluginReleaseStage		finalStage
#define	kTVisualPluginNonFinalRelease	0

enum
{
	kSettingsDialogResID	= 1000,
	
	kSettingsDialogOKButton	= 1,
	kSettingsDialogCancelButton,
	kSettingsDialogUserItem1,
	kSettingsDialogAddButton,
	kSettingsDialogDeleteButton
};

typedef struct VisualPluginData {
	void *				appCookie;
	ITAppProcPtr		appProc;

	ITFileSpec			pluginFileSpec;
	
	CGrafPtr			destPort;
	Rect				destRect;
	OptionBits			destOptions;
	UInt32				destBitDepth;

	RenderVisualData	renderData;
	UInt32				renderTimeStampID;
	
	ITTrackInfo			trackInfo;
	ITStreamInfo		streamInfo;

	Boolean				playing;
	Boolean				padding[3];

//	Plugin-specific data
	UInt8				minLevel[kVisualMaxDataChannels];		// 0-128
	UInt8				maxLevel[kVisualMaxDataChannels];		// 0-128

	GWorldPtr			offscreen;
} VisualPluginData;

/**\
|**|	local (static) globals
\**/

static Boolean	gColorFlag = true;
static Boolean	gFlashFlag = false;

static CGrafPtr	gSavePort;
static GDHandle	gSaveDevice;

static ListHandle	gListHandle = nil;
static Rect			gListRect;

static Str255		gArtistStr;
static Str255		gBadString = "\pBritney Spears";
/**\
|**|	exported function prototypes
\**/

extern OSStatus iTunesPluginMain(OSType message,PluginMessageInfo *messageInfo,void *refCon);

/**\
|**|	static functions
\**/

/*
 Apple event record at: 0E194BB4
  Class: SJAM  ID: Next  Target Type: psn   "iTunes.(HACKED)"
  
  KEY  TYPE LENGTH  DATA
  ---- null 000000  Null object
*/

static OSStatus iTunes_PlayNextTrack(void)
{
	ProcessSerialNumber tPSN = { 0, kCurrentProcess};
	AppleEvent		tAppleEvent = {typeNull,nil};
	OSStatus		anErr;

	anErr = MoreAECreateAppleEventSelfTarget('SJAM','Next',&tAppleEvent);
	if ( anErr == noErr )
	{
		AEDesc 		aliasDesc = {typeNull, nil};

		anErr = AEPutParamDesc( &tAppleEvent, keyDirectObject, &aliasDesc );

		if ( anErr == noErr )
			anErr = MoreAESendEventNoReturnValue( nil, &tAppleEvent );

	    AEDisposeDesc(&tAppleEvent);
	}
	return anErr;
}

/*
 Apple event record at: 0E194BB4
  Class: SJAM  ID: Prev  Target Type: psn   "iTunes.(HACKED)"
  
  KEY  TYPE LENGTH  DATA
  ---- null 000000  Null object
*/

static OSStatus iTunes_PlayPrevTrack(void)
{
	ProcessSerialNumber tPSN = { 0, kCurrentProcess};
	AppleEvent		tAppleEvent = {typeNull,nil};
	OSStatus		anErr;

	anErr = MoreAECreateAppleEventSelfTarget('SJAM','Prev',&tAppleEvent);
	if ( anErr == noErr )
	{
		AEDesc 		aliasDesc = {typeNull, nil};

		anErr = AEPutParamDesc( &tAppleEvent, keyDirectObject, &aliasDesc );

		if ( anErr == noErr )
			anErr = MoreAESendEventNoReturnValue( nil, &tAppleEvent );

		(void) AEDisposeDesc( &tAppleEvent );	// always dispose of AEDescs when you are finished with them
	}
	return anErr;
}

//	MemClear
static void MemClear(LogicalAddress dest,SInt32 length)
{
	register unsigned char	*ptr;

	ptr = (unsigned char*) dest;
	
	while (length-- > 0)
		*ptr++ = 0;
}

// GetPortCopyBitsBitMap
//
static const BitMap* GetPortCopyBitsBitMap(CGrafPtr port)
{
	const BitMap*		destBMap;

#if ACCESSOR_CALLS_ARE_FUNCTIONS
	destBMap = GetPortBitMapForCopyBits(port);
#else
  #if OPAQUE_TOOLBOX_STRUCTS
	PixMapHandle	pixMap;
	
	pixMap		= GetPortPixMap(port);
	destBMap	= (BitMap*) (*pixMap);
  #else
	destBMap	= (BitMap*) &((GrafPtr)port)->portBits;
  #endif
#endif
	return destBMap;
}

// ProcessRenderData

static void ProcessRenderData(VisualPluginData *visualPluginData,const RenderVisualData *renderData)
{
	SInt16		index;
	SInt32		channel;

	if (renderData == nil)
	{
		MemClear(&visualPluginData->renderData,sizeof(visualPluginData->renderData));
		return;
	}

	visualPluginData->renderData = *renderData;
	
	for (channel = 0;channel < renderData->numSpectrumChannels;channel++)
	{
		visualPluginData->minLevel[channel] = 
			visualPluginData->maxLevel[channel] = 
			renderData->spectrumData[channel][0];

		for (index = 1; index < kVisualNumSpectrumEntries; index++)
		{
			UInt8		value;
			
			value = renderData->spectrumData[channel][index];

			if (value < visualPluginData->minLevel[channel])
				visualPluginData->minLevel[channel] = value;
			else if (value > visualPluginData->maxLevel[channel])
				visualPluginData->maxLevel[channel] = value;
		}
	}
}

static const RGBColor gRGBColors[] = {
	{0x0000,0x0000,0x4000},	// dark blue
	{0x4000,0x4000,0xC000},	// lite blue

	{0x0000,0x8000,0x0000},	// green
	{0x4000,0xC000,0x4000},	// lite green
	{0xB000,0x6000,0x4000},	// brown

	{0x8000,0x4000,0x2000},	// dark brown
	{0x4000,0x4000,0x4000},	// gray
	{0xC000,0xC000,0xC000}	// lt gray
};

static void _Erase(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *srcRect,const Rect *destRect,Boolean onlyUpdate)
{
	(visualPluginData); (destPort); (destRect); (onlyUpdate);

	GetGWorld(&gSavePort,&gSaveDevice);

	if (nil != visualPluginData->offscreen)
	{
		SetGWorld(visualPluginData->offscreen,nil);
		ForeColor(yellowColor);
		BackColor(blackColor);

		ClipRect(srcRect);
		EraseRect(srcRect);
	}
}

static void _Draw(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *srcRect,const Rect *destRect,Boolean onlyUpdate)
{
	(onlyUpdate);

	if ((nil == destPort) || (nil == destRect) ||
		(nil == visualPluginData->offscreen))
		return;

//	if (false == onlyUpdate)
	{
		Str255			tMTStr = "\p<EMPTY>";
		Rect			tRect = *srcRect;
		Ptr 			tPtr = (Ptr) &visualPluginData->streamInfo.streamTitle;
		UInt8			tColorIndex;

		tColorIndex = visualPluginData->maxLevel[0] * 8 / 128;
		tColorIndex %= 7;

		tRect.top = (tRect.top + tRect.bottom) >> 2;
		tRect.bottom = tRect.top + 12;

		RGBForeColor(&gRGBColors[tColorIndex]);

		// Stream info
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->streamInfo.streamURL;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->streamInfo.streamMessage;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,(tRect.top + tRect.bottom) >> 2);

		RGBForeColor(&gRGBColors[7 - tColorIndex]);

		// track info
		tPtr = (Ptr) &visualPluginData->trackInfo.name;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.fileName;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.artist;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.album;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.genre;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.kind;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		// misc
		OffsetRect(&tRect,0,20);
		tPtr = (Ptr) &visualPluginData->trackInfo.unusedReserved1;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		

		OffsetRect(&tRect,0,14);
		tPtr = (Ptr) &visualPluginData->trackInfo.comments;
		if (0 == *tPtr) tPtr = (Ptr) &tMTStr;
		TETextBox(tPtr + 1,*tPtr,&tRect,teJustCenter);		
	}
}

static void _Filter(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *srcRect,const Rect *destRect,Boolean onlyUpdate)
{
	(onlyUpdate);	// #pragma unused

	if ((nil != destPort) && (nil != visualPluginData->offscreen))
	{
		const BitMap*		srcBitMap;
		const BitMap*		dstBitMap;

		srcBitMap	= GetPortCopyBitsBitMap(visualPluginData->offscreen);
		dstBitMap	= GetPortCopyBitsBitMap(destPort);

		SetGWorld(destPort,nil);

		ForeColor(blackColor);
		BackColor(whiteColor);

		CopyBits(srcBitMap,dstBitMap,srcRect,destRect,srcCopy,nil);
	}
	SetGWorld(gSavePort,gSaveDevice);
}

/*
	RenderVisualPort
*/
static void RenderVisualPort(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *destRect,Boolean onlyUpdate)
{
#if 1
	(visualPluginData);(destPort);(destRect);(onlyUpdate);	// #pragma unused...
#else
	Rect srcRect = {0,0,0,0};

	if (nil != destRect)
	{
		srcRect = *destRect;
		OffsetRect(&srcRect,-srcRect.left,-srcRect.top);
	}

	_Erase(visualPluginData,destPort,&srcRect,destRect,onlyUpdate);
	_Draw(visualPluginData,destPort,&srcRect,destRect,onlyUpdate);
	_Filter(visualPluginData,destPort,&srcRect,destRect,onlyUpdate);
#endif 0
}

/*
	AllocateVisualData is where you should allocate any information that depends
	on the port or rect changing (like offscreen GWorlds).
*/

static OSStatus AllocateVisualData(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *destRect)
{
	OSStatus		status;
	Rect			allocateRect;

	(void) destPort;

	GetGWorld(&gSavePort,&gSaveDevice);
	
	allocateRect = *destRect;
	OffsetRect(&allocateRect,-allocateRect.left,-allocateRect.top);
				
	status = NewGWorld(&visualPluginData->offscreen,32,&allocateRect,nil,nil,useTempMem);
	if (status == noErr)
	{
		PixMapHandle	pix = GetGWorldPixMap(visualPluginData->offscreen);

		LockPixels(pix);

		// Offscreen starts out black
		SetGWorld(visualPluginData->offscreen,nil);
		
		ForeColor(blackColor);
		PaintRect(&allocateRect);
	}

	SetGWorld(gSavePort,gSaveDevice);
	
	return status;
}

/*
	DeallocateVisualData is where you should deallocate the .
*/
static void DeallocateVisualData(VisualPluginData *visualPluginData)
{
	if (visualPluginData->offscreen != nil)
	{
		DisposeGWorld(visualPluginData->offscreen);
		visualPluginData->offscreen = nil;
	}
}

// ChangeVisualPort
//
static OSStatus ChangeVisualPort(VisualPluginData *visualPluginData,CGrafPtr destPort,const Rect *destRect)
{
	OSStatus		status;
	Boolean			doAllocate;
	Boolean			doDeallocate;
	
	status = noErr;
	
	doAllocate		= false;
	doDeallocate	= false;
		
	if (destPort != nil)
	{
		if (visualPluginData->destPort != nil)
		{
			if (false == EqualRect(destRect,&visualPluginData->destRect))
			{
				doDeallocate	= true;
				doAllocate		= true;
			}
		}
		else
		{
			doAllocate = true;
		}
	}
	else
	{
		doDeallocate = true;
	}
	
	if (doDeallocate)
		DeallocateVisualData(visualPluginData);
	
	if (doAllocate)
		status = AllocateVisualData(visualPluginData,destPort,destRect);

	if (status != noErr)
		destPort = nil;

	visualPluginData->destPort = destPort;
	if (destRect != nil)
		visualPluginData->destRect = *destRect;

	return status;
}

/*
	ResetRenderData
*/
static void ResetRenderData(VisualPluginData *visualPluginData)
{
	MemClear(&visualPluginData->renderData,sizeof(visualPluginData->renderData));

	visualPluginData->minLevel[0] = 
		visualPluginData->minLevel[1] =
		visualPluginData->maxLevel[0] =
		visualPluginData->maxLevel[1] = 0;
}

/*
	SettingsDialogFilterProc
*/
static DEFINE_API(Boolean) SettingsDialogFilterProc(DialogPtr theDialog,EventRecord * theEvent,short * itemHit)
{
	Boolean				handled;
	WindowPtr			theWindow;
	VisualPluginData*	visualPluginData;
	GrafPtr				savePort;
	
	theWindow			= GetDialogWindow(theDialog);
	visualPluginData	= (VisualPluginData*) GetWRefCon(theWindow);
	handled 			= false;
	*itemHit 			= 0;
	
	GetPort(&savePort);
	SetPort((GrafPtr) GetWindowPort(theWindow));
	
	switch (theEvent->what)
	{
	case nullEvent:	// Let iTunes have idle events so the visuals can continue while playing
		PlayerHandleMacOSEvent(visualPluginData->appCookie,visualPluginData->appProc,theEvent,nil);
		break;

	case mouseDown:
		{
			Point localPoint = theEvent->where;
			GlobalToLocal(&localPoint);

			if (PtInRect(localPoint,&gListRect))
				*itemHit = kSettingsDialogUserItem1;

			if (kSettingsDialogUserItem1 == *itemHit)
			{
				if (gListHandle && LClick(localPoint,theEvent->modifiers,gListHandle))
				{
					*itemHit = kSettingsDialogDeleteButton;
				}
				handled = true;
			}
		}
		break;

	case updateEvt:
		{
			if ((WindowPtr) theEvent->message == theWindow)
				if (nil != gListHandle)
					LUpdate((*gListHandle)->port->visRgn,gListHandle);
		}
		break;
	}

	if (false == handled)
	{
		PlayerHandleMacOSEvent(visualPluginData->appCookie,visualPluginData->appProc,theEvent,&handled);
		handled = StdFilterProc(theDialog,theEvent,itemHit);
	}
	SetPort(savePort);
	
	return handled ;
}

static void AddStringToList(ListHandle pListHandle,Str255 pString)
{
	Boolean	found;		// flag variable
	short	numRows;	// number of rows in list
	short	curRow;		// row being examined
	short	cellDataOffset, cellDataLength;	// data being compared to string
	Rect	bounds;
	Cell	aCell;		// cell coordinates}

	found = false;	// initialize flag variable
	bounds = (*pListHandle)->dataBounds;
	numRows = bounds.bottom - bounds.top;

   curRow = -1;	// start before first row

	for (curRow = 0;!found && (curRow < numRows);curRow++)
	{
		Ptr tPtr;

		SetPt(&aCell, 0, curRow);	// prepare to check cell data
		LGetCellDataLocation(&cellDataOffset, &cellDataLength, aCell, pListHandle);
		HLockHi((Handle) (*pListHandle)->cells);    // lock list data in memory
		tPtr = (Ptr) (*(*pListHandle)->cells) + cellDataOffset;

//		DebugStr("\p|strncmp;");
		if (0 == strncmp((Ptr) &pString[1],tPtr,pString[0]))
			found = true;
         HUnlock((Handle) (*pListHandle)->cells);    // unlock list data
	}
	if (found)
		return;

	curRow = LAddRow(1, curRow, pListHandle);
	SetPt(&aCell, 0, curRow);	// prepare to set cell data
	// set data
	LSetCell(&pString[1], pString[0], aCell,pListHandle);          
}

/* 
	DoSettingsDialog
*/
static OSStatus DoSettingsDialog(VisualPluginData *visualPluginData)
{
	OSStatus		status;
	DialogPtr		theDialog;
	GrafPtr			savePort;
	
	status = noErr;
	
	GetPort(&savePort);
	
	theDialog = GetNewDialog(kSettingsDialogResID,nil,(WindowRef) -1L);
	if(theDialog != nil)
	{
		WindowRef		theWindow;
		ControlHandle	theControl;
		ModalFilterUPP	filterUPP;
		SInt16			itemHit;

		theWindow = GetDialogWindow(theDialog);

		SetPort((GrafPtr) GetWindowPort(theWindow));
		
		SetWRefCon(theWindow,(UInt32) visualPluginData);

		SetDialogDefaultItem(theDialog,kSettingsDialogOKButton);
		SetDialogCancelItem(theDialog,kSettingsDialogCancelButton);

		status = GetDialogItemAsControl(theDialog,kSettingsDialogUserItem1,&theControl);
//		if (GetDialogItemAsControl(theDialog,kSettingsDialogUserItem1,&theControl) == noErr)
		if ((noErr == status) && (nil != theControl))
		{
			Point	tSize = {0,0};
			ListBounds tListBounds = {0,0,0,1};
//			UInt32 actualSize;

			GetControlBounds(theControl,&gListRect);

			gListRect.right -= 15;	// Make room for scroll bar
			gListHandle = LNew(&gListRect,&tListBounds,tSize,0,theWindow,true,false,false,true);

//			DebugStr("\p|GetControlProperty;");
//			status = GetControlProperty(theControl,kTVisualPluginCreator,kControlListBoxListHandleTag,sizeof(gListHandle),&actualSize,(void*) &gListHandle);
//			if (noErr != status)
//				DebugStr("\p|GetControlProperty error;");

			if (nil != gListHandle)
			{
				AddStringToList(gListHandle,"\pOne");
				AddStringToList(gListHandle,"\pTwo");
				AddStringToList(gListHandle,"\pThree");
				AddStringToList(gListHandle,"\pTwo");
				AddStringToList(gListHandle,"\pOne");

				InvalRect(&(*gListHandle)->rView);
				if (nil != (*gListHandle)->vScroll)
				   InvalRect(&(*(*gListHandle)->vScroll)->contrlRect);
				if (nil != (*gListHandle)->hScroll)
				   InvalRect(&(*(*gListHandle)->hScroll)->contrlRect);
			}
		}

		ShowWindow(theWindow);
		SelectWindow(theWindow);

		itemHit = 0;
		
		filterUPP = NewModalFilterUPP(SettingsDialogFilterProc);
		
		while ((itemHit != kSettingsDialogOKButton) && (itemHit != kSettingsDialogCancelButton))
		{
			ModalDialog(filterUPP,&itemHit);
		
			switch (itemHit)
			{
			case kSettingsDialogAddButton:
				if (gArtistStr[0])
					AddStringToList(gListHandle,gArtistStr);
				break;
			case kSettingsDialogDeleteButton:
			//	DebugStr("\p|Hit kSettingsDialogDeleteButton!;");
				while (true)
				{
					Cell theCell = {0,0};

					if (!LGetSelect(true,&theCell,gListHandle))
						break;
			//	DebugStr("\p|Got List Cell!;");
					LDelRow(1,theCell.v,gListHandle);
				}
				break;

			case kSettingsDialogCancelButton:
				status = userCanceledErr;
				break;
				
			default:
				break;
			}
		}
		DisposeModalFilterUPP(filterUPP);
		DisposeDialog(theDialog);
	}
	else
		status = memFullErr;

	SetPort(savePort);
	
	return status;
}

static void Copy_Info(const OSType message,VisualPluginMessageInfo *messageInfo,VisualPluginData  *visualPluginData)
{
	ITTrackInfo*	trackInfo;
	ITStreamInfo*	streamInfo;

	if ((nil == messageInfo) || (nil == visualPluginData))
		return;

	switch (message)
	{
	case kVisualPluginChangeTrackMessage:
		trackInfo = messageInfo->u.changeTrackMessage.trackInfo;
		streamInfo = messageInfo->u.changeTrackMessage.streamInfo;
		break;
	case kVisualPluginPlayMessage:
		trackInfo = messageInfo->u.playMessage.trackInfo;
		streamInfo = messageInfo->u.playMessage.streamInfo;
		break;
	default:
		return;
	}

	if (trackInfo != nil)
	{
		visualPluginData->trackInfo = *trackInfo;
		if ((visualPluginData->trackInfo.validFields & kITTIArtistFieldMask) == kITTIArtistFieldMask)
		{
			if (0 != visualPluginData->trackInfo.artist[0])
				PLstrcpy(gArtistStr,visualPluginData->trackInfo.artist);

			if (0 == PLstrcmp(visualPluginData->trackInfo.artist,gBadString))
			{
//				DebugStr("\p|iTunes_PlayNextTrack;g");
				iTunes_PlayNextTrack();
			}
		}
	}
	else
		MemClear(&visualPluginData->trackInfo,sizeof(visualPluginData->trackInfo));

	if (streamInfo != nil)
		visualPluginData->streamInfo = *streamInfo;
	else
		MemClear(&visualPluginData->streamInfo,sizeof(visualPluginData->streamInfo));
}

/*
	VisualPluginHandler
*/
static OSStatus VisualPluginHandler(OSType message,VisualPluginMessageInfo *messageInfo,void *refCon)
{
	OSStatus			status;
	VisualPluginData *	visualPluginData;
	
	visualPluginData = (VisualPluginData*) refCon;
	
	status = noErr;

	switch (message)
	{
		/*
			Sent when the visual plugin is registered.  The plugin should do minimal
			memory allocations here.  The resource fork of the plugin is still available.
		*/		
		case kVisualPluginInitMessage:
		{
			// DebugStr("\p|kVisualPluginInitMessage-I-Debug;g");

			visualPluginData = (VisualPluginData*) NewPtrClear(sizeof(VisualPluginData));
			if (visualPluginData == nil)
			{
				status = memFullErr;
				break;
			}

			visualPluginData->appCookie	= messageInfo->u.initMessage.appCookie;
			visualPluginData->appProc	= messageInfo->u.initMessage.appProc;

			/* Remember the file spec of our plugin file. We need this so we can open our resource fork during */
			/* the configuration message */
			
			status = PlayerGetPluginFileSpec(visualPluginData->appCookie,visualPluginData->appProc,&visualPluginData->pluginFileSpec);

			messageInfo->u.initMessage.refCon	= (void*) visualPluginData;
			break;
		}
			
		/*
			Sent when the visual plugin is unloaded
		*/		
		case kVisualPluginCleanupMessage:
			// DebugStr("\p|kVisualPluginInitMessage-I-Debug;g");

			if (visualPluginData != nil)
				DisposePtr((Ptr)visualPluginData);
				visualPluginData = nil;
			break;
			
		/*
			Sent when the visual plugin is enabled.  iTunes currently enables all
			loaded visual plugins.  The plugin should not do anything here.
		*/
		case kVisualPluginEnableMessage:
			// DebugStr("\p|kVisualPluginEnableMessage-I-Debug;g");
			break;
		case kVisualPluginDisableMessage:
			// DebugStr("\p|kVisualPluginEnableMessage-I-Debug;g");
			break;

		/*
			Sent if the plugin requests idle messages.  Do this by setting the kVisualWantsIdleMessages
			option in the RegisterVisualMessage.options field.
		*/
		case kVisualPluginIdleMessage:
//			// DebugStr("\p|kVisualPluginIdleMessage-I-Debug;g");
			if (false == visualPluginData->playing)
				RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,false);
			break;
					
		/*
			Sent if the plugin requests the ability for the user to configure it.  Do this by setting
			the kVisualWantsConfigure option in the RegisterVisualMessage.options field.
		*/
		case kVisualPluginConfigureMessage:
		{
			short	oldResFile;
			short	ourResFile;

			// DebugStr("\p|kVisualPluginConfigureMessage-I-Debug;g");
			
			oldResFile = CurResFile();
			
			ourResFile = FSpOpenResFile(&visualPluginData->pluginFileSpec,fsRdPerm);
			if (ourResFile > 0)
			{
				status = DoSettingsDialog(visualPluginData);
				CloseResFile(ourResFile);
			}
			else
				status = fnfErr;

			UseResFile(oldResFile);
			break;
		}
		
		/*
			Sent when iTunes is going to show the visual plugin in a port.  At
			this point,the plugin should allocate any large buffers it needs.
		*/
		case kVisualPluginShowWindowMessage:
			// DebugStr("\p|kVisualPluginShowWindowMessage-I-Debug;g");
			visualPluginData->destOptions = messageInfo->u.showWindowMessage.options;

			status = ChangeVisualPort(	visualPluginData,
										messageInfo->u.showWindowMessage.port,
										&messageInfo->u.showWindowMessage.drawRect);
			if (status == noErr)
				RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,true);
			break;
			
		/*
			Sent when iTunes is no longer displayed.
		*/
		case kVisualPluginHideWindowMessage:
			// DebugStr("\p|kVisualPluginHideWindowMessage-I-Debug;g");
			(void) ChangeVisualPort(visualPluginData,nil,nil);

			MemClear(&visualPluginData->trackInfo,sizeof(visualPluginData->trackInfo));
			MemClear(&visualPluginData->streamInfo,sizeof(visualPluginData->streamInfo));
			break;
		
		/*
			Sent when iTunes needs to change the port or rectangle of the currently
			displayed visual.
		*/
		case kVisualPluginSetWindowMessage:
			// DebugStr("\p|kVisualPluginSetWindowMessage-I-Debug;g");
			visualPluginData->destOptions = messageInfo->u.setWindowMessage.options;

			status = ChangeVisualPort(	visualPluginData,
										messageInfo->u.setWindowMessage.port,
										&messageInfo->u.setWindowMessage.drawRect);

			if (status == noErr)
				RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,true);
			break;
		
		/*
			Sent for the visual plugin to render a frame.
		*/
		case kVisualPluginRenderMessage:
//			// DebugStr("\p|kVisualPluginRenderMessage-I-Debug;g");
			visualPluginData->renderTimeStampID	= messageInfo->u.renderMessage.timeStampID;

			ProcessRenderData(visualPluginData,messageInfo->u.renderMessage.renderData);
				
			RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,false);
			break;

		/*
			Sent in response to an update event.  The visual plugin should update
			into its remembered port.  This will only be sent if the plugin has been
			previously given a ShowWindow message.
		*/	
		case kVisualPluginUpdateMessage:
			// DebugStr("\p|kVisualPluginUpdateMessage-I-Debug;g");
			RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,true);
			break;
		
		/*
			Sent when the player starts.
		*/
		case kVisualPluginPlayMessage:
			// DebugStr("\p|kVisualPluginPlayMessage-I-Debug;g");
			Copy_Info(message,messageInfo,visualPluginData);
		
			visualPluginData->playing = true;
			break;

		/*
			Sent when the player changes the current track information.  This
			is used when the information about a track changes,or when the CD
			moves onto the next track.  The visual plugin should update any displayed
			information about the currently playing song.
		*/
		case kVisualPluginChangeTrackMessage:
			// DebugStr("\p|kVisualPluginChangeTrackMessage-I-Debug;g");
			Copy_Info(message,messageInfo,visualPluginData);
			break;

		/*
			Sent when the player stops.
		*/
		case kVisualPluginStopMessage:
			// DebugStr("\p|kVisualPluginStopMessage-I-Debug;g");
			visualPluginData->playing = false;
			
			ResetRenderData(visualPluginData);

			RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,true);
			break;
		
		/*
			Sent when the player changes position.
		*/
		case kVisualPluginSetPositionMessage:
			// DebugStr("\p|kVisualPluginSetPositionMessage-I-Debug;g");
			break;

		/*
			Sent when the player pauses.  iTunes does not currently use pause or unpause.
			A pause in iTunes is handled by stopping and remembering the position.
		*/
		case kVisualPluginPauseMessage:
			// DebugStr("\p|kVisualPluginPauseMessage-I-Debug;g");
			visualPluginData->playing = false;

			ResetRenderData(visualPluginData);

			RenderVisualPort(visualPluginData,visualPluginData->destPort,&visualPluginData->destRect,true);
			break;
			
		/*
			Sent when the player unpauses.  iTunes does not currently use pause or unpause.
			A pause in iTunes is handled by stopping and remembering the position.
		*/
		case kVisualPluginUnpauseMessage:
			// DebugStr("\p|kVisualPluginUnpauseMessage-I-Debug;g");
			visualPluginData->playing = true;
			break;
		
		/*
			Sent to the plugin in response to a MacOS event.  The plugin should return noErr
			for any event it handles completely,or an error (unimpErr) if iTunes should handle it.
		*/
		case kVisualPluginEventMessage:
			// DebugStr("\p|kVisualPluginEventMessage-I-Debug;g");
			{
				EventRecord* tEventPtr = messageInfo->u.eventMessage.event;
				if ((tEventPtr->what == keyDown) || (tEventPtr->what == autoKey))
				{    // charCodeMask,keyCodeMask;
					char theChar = tEventPtr->message & charCodeMask;

					status = noErr;

					switch (theChar)
					{
					case	'+':
						//AddStringToList(gListHandle,"\pNine");
						iTunes_PlayNextTrack();
						break;
					case	'-':
						iTunes_PlayPrevTrack();
						break;
					default:
						status = unimpErr;
						break;
					}
				}
				else
					status = unimpErr;
			}
			break;

		default:
			// DebugStr("\p|kVisualPlugin[UNKNOWN]Message-I-Debug;g");
			status = unimpErr;
			break;
	}
	return status;	
}

/*
	RegisterVisualPlugin
*/
static OSStatus RegisterVisualPlugin(PluginMessageInfo *messageInfo)
{
	OSStatus			status;
	PlayerMessageInfo	playerMessageInfo;
	Str255				pluginName = kTVisualPluginName;
		
	MemClear(&playerMessageInfo.u.registerVisualPluginMessage,sizeof(playerMessageInfo.u.registerVisualPluginMessage));
	
	BlockMoveData((Ptr)&pluginName[0],(Ptr)&playerMessageInfo.u.registerVisualPluginMessage.name[0],pluginName[0] + 1);

	SetNumVersion(&playerMessageInfo.u.registerVisualPluginMessage.pluginVersion,kTVisualPluginMajorVersion,kTVisualPluginMinorVersion,kTVisualPluginReleaseStage,kTVisualPluginNonFinalRelease);

//	playerMessageInfo.u.registerVisualPluginMessage.options					= kVisualWantsIdleMessages | kVisualWantsConfigure;
	playerMessageInfo.u.registerVisualPluginMessage.options					= kVisualWantsIdleMessages;
	playerMessageInfo.u.registerVisualPluginMessage.handler					= VisualPluginHandler;
	playerMessageInfo.u.registerVisualPluginMessage.registerRefCon			= 0;
	playerMessageInfo.u.registerVisualPluginMessage.creator					= kTVisualPluginCreator;
	
	playerMessageInfo.u.registerVisualPluginMessage.timeBetweenDataInMS		= 0xFFFFFFFF; // 16 milliseconds = 1 Tick,0xFFFFFFFF = Often as possible.
//	playerMessageInfo.u.registerVisualPluginMessage.timeBetweenDataInMS		= 1000;
	playerMessageInfo.u.registerVisualPluginMessage.numWaveformChannels		= 2;
	playerMessageInfo.u.registerVisualPluginMessage.numSpectrumChannels		= 2;
	
	playerMessageInfo.u.registerVisualPluginMessage.minWidth				= 64;
	playerMessageInfo.u.registerVisualPluginMessage.minHeight				= 64;
	playerMessageInfo.u.registerVisualPluginMessage.maxWidth				= 32767;
	playerMessageInfo.u.registerVisualPluginMessage.maxHeight				= 32767;
	playerMessageInfo.u.registerVisualPluginMessage.minFullScreenBitDepth	= 0;
	playerMessageInfo.u.registerVisualPluginMessage.maxFullScreenBitDepth	= 0;
	playerMessageInfo.u.registerVisualPluginMessage.windowAlignmentInBytes	= 0;
	
	status = PlayerRegisterVisualPlugin(messageInfo->u.initMessage.appCookie,messageInfo->u.initMessage.appProc,&playerMessageInfo);
		
	return status;
}

/**\
|**|	main entrypoint
\**/

OSStatus iTunesPluginMain(OSType message,PluginMessageInfo *messageInfo,void *refCon)
{
	OSStatus		status;
	
	(void) refCon;
	
	switch (message)
	{
		case kPluginInitMessage:
			status = RegisterVisualPlugin(messageInfo);
			break;
			
		case kPluginCleanupMessage:
			status = noErr;
			break;
			
		default:
			status = unimpErr;
			break;
	}
	return status;
}
