/*	SpellFile 
	- class contains all methods necessary to carry out all transactions between
		the file system and the SpellBook application.
		
	05.01.99 by Scott C. Ziegler for use with AD&D and Simulcra RPG.
	
	vers. history:
		
		05.23.00 - Illuminatus! Day. Moved from 2 methods to 4 by abstracting the actual
					disk operations code from the JFileChooser display.  This allows for
					calls to save the state of the spellbook without prompting the user.
					Benefits include default/autosave capability.
		06.23.00 - Illuminatus! Day. achieved syntacticallly correct version. (@ MacHack)
		02.19.01 - changed some parameter interactions such as load doesn't need a spellbook param
	
	Work Needed:
		- !!! Major Error in Design: SpellBook, when updated will invalidate previously
			serialized SpellBooks of earlier version.  It is smarter to have a special
			format that writes several java objects like ints, vectors, & strings.
		- need to make a backup method that keeps a separate record, char based file (gzipped)
		- add more accessory functions like wrappered autosaves, etc.
		- add import function which takes a tab-delineated text file and converts
			it to a SpellRecord
		
	Work Completed:
		
		- added the parent parameter for save as and load from so FileChooser can show dialog.
		- changed load parameters
	
*/

/*
This file and its intellectual contents are considered property of the creator and are to be treated as such with respect to use in other applications.  Any use of this software for any purpose whatsoever must have explicit permission from the author of the file.  This code is part of an ongoing development process for future possible products, and so is considered private property.
Do not use without permission.  Author: Scott C. Ziegler <Aslan@Narnia.net>
*/

package Arcana;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.JFileChooser;

public class SpellFile {

	static final String DEFAULT_FILENAME = "untitled.spellbook";

	File  	spellFile;
	String	currentPath;
	
	SpellBook	storedBook;
	
	public SpellFile() {
		spellFile = new File(DEFAULT_FILENAME);
		storedBook = new SpellBook();
		}
	
	public SpellFile(String path) {
		spellFile = new File(path);
		currentPath = path;
		storedBook = new SpellBook();
		}
	
	// Methods:
	
	public void setPath(String path) {
		currentPath = path;
		}
		
	public String getPath() {
		return currentPath;
		}
	
	public SpellBook load() {
		
	//	if (currentPath.compareTo("") == 0) load(DEFAULT_FILENAME);
	//	else load(currentPath);
		
		return load(((currentPath.compareTo("") == 0)?DEFAULT_FILENAME:currentPath));
		}
	
	public SpellBook load(String filename) {
		Vector readVector = new Vector();
		if (filename != null) {
			try {
				FileInputStream fis = new FileInputStream(filename);
				//GZIPInputStream gzis = new GZIPInputStream(fis);
				//ObjectInputStream in = new ObjectInputStream(gzis);
				ObjectInputStream in = new ObjectInputStream(fis);
				
				readVector = (Vector)in.readObject();
				storedBook = (SpellBook)readVector.elementAt(0);
				in.close();
				return storedBook;
				
				}
			catch (IOException ie) {
				System.out.println("Error -- " + ie.toString());
				return storedBook = new SpellBook();
				}
			catch (ClassNotFoundException cnfe) {
				return storedBook = new SpellBook();
				}
			}
		else {
			// throw a JDialog w/ error. (actually JOptionPane)
			return storedBook = new SpellBook();
			}
		}
		
	public void save(SpellBook spellBook) {
		if (currentPath.compareTo("") == 0) save(DEFAULT_FILENAME, spellBook);
		else save(currentPath, spellBook);
		}
	
	public void save(String filename, SpellBook spellBook) {
		Vector writeVector = new Vector();
		if (filename != null) {
			try {
				FileOutputStream fos = new FileOutputStream(filename);
				//GZIPOutputStream gzos = new GZIPOutputStream(fos);
				//ObjectOutputStream out = new ObjectOutputStream(gzos);
				ObjectOutputStream out = new ObjectOutputStream(fos);
				
				writeVector.addElement(spellBook);
				out.writeObject(writeVector);
				out.flush();
				out.close();
				}
			catch (IOException ie) {
				System.out.println("Error -- " + ie.toString());
				}
			}
		else {
			// throw a JDialog w/ error. (actually JOptionPane)
			}
		}
	
	public SpellBook loadFrom(Component parent) {
		JFileChooser f = new JFileChooser();
		f.showOpenDialog(parent);
		String filename = f.getName();
		return load(filename); 
		}
		
	public void saveAs(SpellBook spellBook, Component parent) {
		JFileChooser f = new JFileChooser();
		f.setDialogType(JFileChooser.SAVE_DIALOG);
		f.setDialogTitle("Save Spells");
		f.showSaveDialog(parent);
		String filename = f.getSelectedFile().getName();
		save(filename, spellBook);
		}
	}