/*
	
	TECalc
	
	Joshua Juran
	
*/

#include <string.h>

#include <Events.h>
#include <NumberFormatting.h>
#include <Resources.h>
#include <Sound.h>
#include <TextEdit.h>

// Needed for globals
#include <A4Stuff.h>

#define _TEKey 0xA9DC

extern long expression(const char *inExpr);

static pascal void PatchedTEKey(short c, TEHandle hTE);

typedef pascal void (*TEKeyProcPtr)(short, TEHandle);
static TEKeyProcPtr	origTEKey;

static void Payload(TEHandle hTE)
{
	long start, end, len;
	char buf[256];
	long value;
	
	start = (**hTE).selStart;
	end   = (**hTE).selEnd;
	len = end - start;
	
	if (len > 255) {
		SysBeep(0);
		return;
	}
	
	memcpy(buf, *(**hTE).hText, len);
	buf[len] = '\0';
	value = expression(buf);
	NumToString((short)value, (unsigned char *)buf);
	
	TEDelete(hTE);
	TEInsert(buf + 1, buf[0], hTE);
	TESetSelect(start, start + buf[0], hTE);
}

static pascal void PatchedTEKey(short c, TEHandle hTE)
{
	long oldA4;
	long start, end;
	
	oldA4 = SetCurrentA4();
	
	start = (**hTE).selStart;
	end   = (**hTE).selEnd;
	
	if ((c == '=') && (start != end)) {
		Payload(hTE);
	} else {
		origTEKey(c, hTE);
	}
	
	SetA4(oldA4);
}

static ProcPtr ApplyTrapPatch(short trap, ProcPtr patchPtr)
{
	ProcPtr trapPtr;
	
	if (patchPtr == nil)
		return nil;

	trapPtr = NGetTrapAddress (trap, (trap & 0x0800) ? ToolTrap : OSTrap);
	NSetTrapAddress (patchPtr, trap, (trap & 0x0800) ? ToolTrap : OSTrap);
	return trapPtr;
}

void main(void)
{
	long oldA4;
	UniversalProcPtr procPtr;
	Handle	initCode = nil;
	THz		oldZone;
	
	oldA4 = SetCurrentA4();
	
	oldZone = GetZone();
	SetZone(SystemZone());
	
	initCode = (Handle)Get1Resource('INIT', 0);
	if (initCode == nil)
		goto failure;
	
	DetachResource(initCode);
	if (ResError() != noErr) 
		goto failure;
	
	HLockHi(initCode);
	HNoPurge(initCode);
	
	procPtr = ApplyTrapPatch( _TEKey, (ProcPtr)(PatchedTEKey) );
	origTEKey = (TEKeyProcPtr)procPtr;
	
failure:
	SetZone(oldZone);
	SetA4(oldA4);
}
